@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: cli.py 1181 2007-03-25 16:18:22Z pronovic $
+# Revision : $Id: cli.py 825 2007-12-19 04:06:57Z pronovic $
 # Purpose  : Provides command-line interface implementation.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -88,8 +88,10 @@ import getopt
 # Cedar Backup modules
 from CedarBackup2.release import AUTHOR, EMAIL, VERSION, DATE, COPYRIGHT
 from CedarBackup2.util import RestrictedContentList, DirectedGraph, PathResolverSingleton
-from CedarBackup2.util import sortDict, splitCommandLine, executeCommand, getFunctionReference, getUidGid, encodePath
+from CedarBackup2.util import sortDict, splitCommandLine, executeCommand, getFunctionReference
+from CedarBackup2.util import getUidGid, encodePath
 from CedarBackup2.config import Config
+from CedarBackup2.peer import RemotePeer
 from CedarBackup2.actions.collect import executeCollect
 from CedarBackup2.actions.stage import executeStage
 from CedarBackup2.actions.store import executeStore
@@ -128,10 +130,11 @@ VALID_ACTIONS      = [ "collect", "stage", "store", "purge", "rebuild", "validat
 COMBINE_ACTIONS    = [ "collect", "stage", "store", "purge", ]
 NONCOMBINE_ACTIONS = [ "rebuild", "validate", "initialize", "all", ]
 
-SHORT_SWITCHES     = "hVbqc:fl:o:m:Ods"
+SHORT_SWITCHES     = "hVbqc:fMNl:o:m:Ods"
 LONG_SWITCHES      = [ 'help', 'version', 'verbose', 'quiet', 
-                       'config=', 'full', 'logfile=', 'owner=', 
-                       'mode=', 'output', 'debug', 'stack', ]
+                       'config=', 'full', 'managed', 'managed-only',
+                       'logfile=', 'owner=', 'mode=', 
+                       'output', 'debug', 'stack', ]
 
 
 #######################################################################
@@ -223,11 +226,22 @@ def cli():
       logger.debug("Using user-supplied configuration file.")
       configPath = options.config
 
+   executeLocal = True
+   executeManaged = False
+   if options.managedOnly:
+      executeLocal = False
+      executeManaged = True
+   if options.managed:
+      executeManaged = True
+   logger.debug("Execute local actions: %s" % executeLocal)
+   logger.debug("Execute managed actions: %s" % executeManaged)
+
    try:
       logger.info("Configuration path is [%s]" % configPath)
       config = Config(xmlPath=configPath)
       setupPathResolver(config)
-      actionSet = _ActionSet(options.actions, config.extensions, config.options.hooks)
+      actionSet = _ActionSet(options.actions, config.extensions, config.options, 
+                             config.peers, executeManaged, executeLocal)
    except Exception, e:
       logger.error("Error reading or handling configuration: %s" % e)
       logger.info("Cedar Backup run completed with status 4.")
@@ -277,11 +291,16 @@ class _ActionItem(object):
    (if set).
 
    @note: The comparison operators for this class have been implemented to only
-   compare based on the index value, and ignore all other values.  This is so
-   that the action set list can be easily sorted by index.
+   compare based on the index and SORT_ORDER value, and ignore all other
+   values.  This is so that the action set list can be easily sorted first by
+   type (_ActionItem before _ManagedActionItem) and then by index within type.
+
+   @cvar SORT_ORDER: Defines a sort order to order properly between types.
 
    @sort: __init__, index, module, function 
    """
+   
+   SORT_ORDER = 0
 
    def __init__(self, index, name, preHook, postHook, function):
       """
@@ -316,6 +335,12 @@ class _ActionItem(object):
             return -1
          else:
             return 1 
+      else:
+         if self.SORT_ORDER != other.SORT_ORDER:
+            if self.SORT_ORDER < other.SORT_ORDER:
+               return -1
+            else:
+               return 1 
       return 0
 
    def executeAction(self, configPath, options, config):
@@ -359,6 +384,84 @@ class _ActionItem(object):
       executeCommand(command=fields[0:1], args=fields[1:])
 
 
+###########################
+# _ManagedActionItem class
+###########################
+
+class _ManagedActionItem(object):
+
+   """
+   Class representing a single action to be executed on a managed peer.
+
+   This class represents a single named action to be executed, and understands
+   how to execute that action.   
+
+   Actions to be executed on a managed peer rely on peer configuration and
+   on the full-backup flag.  All other configuration takes place on the remote
+   peer itself.
+
+   @note: The comparison operators for this class have been implemented to only
+   compare based on the index and SORT_ORDER value, and ignore all other
+   values.  This is so that the action set list can be easily sorted first by
+   type (_ActionItem before _ManagedActionItem) and then by index within type.
+
+   @cvar SORT_ORDER: Defines a sort order to order properly between types.
+   """
+
+   SORT_ORDER = 1
+
+   def __init__(self, index, name, remotePeers):
+      """
+      Default constructor.
+
+      @param index: Index of the item (or C{None}).
+      @param name: Name of the action that is being executed.
+      @param remotePeers: List of remote peers on which to execute the action.
+      """
+      self.index = index
+      self.name = name
+      self.remotePeers = remotePeers
+
+   def __cmp__(self, other):
+      """
+      Definition of equals operator for this class.
+      The only thing we compare is the item's index.  
+      @param other: Other object to compare to.
+      @return: -1/0/1 depending on whether self is C{<}, C{=} or C{>} other.
+      """
+      if other is None:
+         return 1
+      if self.index != other.index:
+         if self.index < other.index:
+            return -1
+         else:
+            return 1 
+      else:
+         if self.SORT_ORDER != other.SORT_ORDER:
+            if self.SORT_ORDER < other.SORT_ORDER:
+               return -1
+            else:
+               return 1 
+      return 0
+
+   def executeAction(self, configPath, options, config):
+      """
+      Executes the managed action associated with an item.
+
+      @note: Only options.full is actually used.  The rest of
+      the arguments exist to satisfy the ActionItem iterface.
+
+      @param configPath: Path to configuration file on disk.
+      @param options: Command-line options to be passed to action.
+      @param config: Parsed configuration to be passed to action.
+
+      @raise Exception: If there is a problem executing the action.
+      """
+      for peer in self.remotePeers:
+         logger.debug("Executing managed action [%s] on peer [%s]." % (self.name, peer.name))
+         peer.executeManagedAction(self.name, options.full)
+
+
 ###################
 # _ActionSet class
 ###################
@@ -366,9 +469,9 @@ class _ActionItem(object):
 class _ActionSet(object):
 
    """
-   Class representing a set of actions to be executed.
+   Class representing a set of local actions to be executed.
 
-   This class does three different things.  First, it ensures that the actions
+   This class does four different things.  First, it ensures that the actions
    specified on the command-line are sensible.  The command-line can only list
    either built-in actions or extended actions specified in configuration.
    Also, certain actions (in L{NONCOMBINE_ACTIONS}) cannot be combined with
@@ -384,10 +487,13 @@ class _ActionSet(object):
    are executed immediately before their associated action, and post-action
    hooks are executed immediately after their associated action.
 
+   Finally, the class properly interleaves local and managed actions so that
+   the same action gets executed first locally and then on managed peers.
+
    @sort: __init__, executeActions
    """
 
-   def __init__(self, actions, extensions, hooks):
+   def __init__(self, actions, extensions, options, peers, managed, local):
       """
       Constructor for the C{_ActionSet} class.
 
@@ -400,25 +506,31 @@ class _ActionSet(object):
          - C{preHookMap}: Mapping from action name to post C{ActionHook}
          - C{functionMap}: Mapping from action name to Python function
          - C{indexMap}: Mapping from action name to execution index
+         - C{peerMap}: Mapping from action name to set of C{RemotePeer}
          - C{actionMap}: Mapping from action name to C{_ActionItem}
 
       Once these data structures are set up, the command line is validated to
       make sure only valid actions have been requested, and in a sensible
       combination.  Then, all of the data is used to build C{self.actionSet},
-      the set of C{_ActionItem} object to be executed by C{executeActions()}.
+      the set action items to be executed by C{executeActions()}.  This list
+      might contain either C{_ActionItem} or C{_ManagedActionItem}.
 
       @param actions: Names of actions specified on the command-line.
       @param extensions: Extended action configuration (i.e. config.extensions)
-      @param hooks: List of pre- and post-action hooks (i.e. config.options.hooks)
+      @param options: Options configuration (i.e. config.options)
+      @param peers: Peers configuration (i.e. config.peers)
+      @param managed: Whether to include managed actions in the set
+      @param local: Whether to include local actions in the set
 
       @raise ValueError: If one of the specified actions is invalid.
-      @raise ValueError: 
       """
       extensionNames = _ActionSet._deriveExtensionNames(extensions)
-      (preHookMap, postHookMap) = _ActionSet._buildHookMaps(hooks)
+      (preHookMap, postHookMap) = _ActionSet._buildHookMaps(options.hooks)
       functionMap = _ActionSet._buildFunctionMap(extensions)
       indexMap = _ActionSet._buildIndexMap(extensions)
-      actionMap = _ActionSet._buildActionMap(extensionNames, functionMap, indexMap, preHookMap, postHookMap)
+      peerMap = _ActionSet._buildPeerMap(options, peers)
+      actionMap = _ActionSet._buildActionMap(managed, local, extensionNames, functionMap, 
+                                             indexMap, preHookMap, postHookMap, peerMap)
       _ActionSet._validateActions(actions, extensionNames)
       self.actionSet = _ActionSet._buildActionSet(actions, actionMap)
 
@@ -559,22 +671,30 @@ class _ActionSet(object):
       return indexMap
    _buildIndexMap = staticmethod(_buildIndexMap)
 
-   def _buildActionMap(extensionNames, functionMap, indexMap, preHookMap, postHookMap):
+   def _buildActionMap(managed, local, extensionNames, functionMap, indexMap, preHookMap, postHookMap, peerMap):
       """
-      Builds a mapping from action name to list of C{_ActionItem} objects.
+      Builds a mapping from action name to list of action items.
 
-      In most cases, the mapping from action name to C{_ActionItem} is 1:1.  The exception
-      is the "all" action, which is a special case.  However, a list is returned in all
-      cases, just for consistency later.
+      We build either C{_ActionItem} or C{_ManagedActionItem} objects here.  
 
-      Each C{_ActionItem} will be created with a proper function reference and
-      index value for execution ordering.
+      In most cases, the mapping from action name to C{_ActionItem} is 1:1.
+      The exception is the "all" action, which is a special case.  However, a
+      list is returned in all cases, just for consistency later.  Each
+      C{_ActionItem} will be created with a proper function reference and index
+      value for execution ordering.
 
+      The mapping from action name to C{_ManagedActionItem} is always 1:1.
+      Each managed action item contains a list of peers which the action should
+      be executed.
+
+      @param managed: Whether to include managed actions in the set
+      @param local: Whether to include local actions in the set
       @param extensionNames: List of valid extended action names 
       @param functionMap: Dictionary mapping action name to Python function
       @param indexMap: Dictionary mapping action name to integer execution index
       @param preHookMap: Dictionary mapping action name to pre hooks (if any) for the action
       @param postHookMap: Dictionary mapping action name to post hooks (if any) for the action
+      @param peerMap: Dictionary mapping action name to list of remote peers on which to execute the action
 
       @return: Dictionary mapping action name to list of C{_ActionItem} objects.
       """
@@ -583,12 +703,49 @@ class _ActionSet(object):
          if name != 'all': # do this one later
             function = functionMap[name]
             index = indexMap[name]
-            (preHook, postHook) = _ActionSet._deriveHooks(name, preHookMap, postHookMap)
-            actionMap[name] = [ _ActionItem(index, name, preHook, postHook, function), ]
+            actionMap[name] = []
+            if local:
+               (preHook, postHook) = _ActionSet._deriveHooks(name, preHookMap, postHookMap)
+               actionMap[name].append(_ActionItem(index, name, preHook, postHook, function))
+            if managed:
+               if name in peerMap:
+                  actionMap[name].append(_ManagedActionItem(index, name, peerMap[name]))
       actionMap['all'] = actionMap['collect'] + actionMap['stage'] + actionMap['store'] + actionMap['purge']
       return actionMap
    _buildActionMap = staticmethod(_buildActionMap)
 
+   def _buildPeerMap(options, peers):
+      """
+      Build a mapping from action name to list of remote peers.
+
+      There will be one entry in the mapping for each managed action.  If there
+      are no managed peers, the mapping will be empty.  Only managed actions
+      will be listed in the mapping.
+
+      @param options: Option configuration (i.e. config.options)
+      @param peers: Peers configuration (i.e. config.peers)
+      """
+      peerMap = {}
+      if peers is not None:
+         if peers.remotePeers is not None:
+            for peer in peers.remotePeers:
+               if peer.managed:
+                  remoteUser = _ActionSet._getRemoteUser(options, peer)
+                  rshCommand = _ActionSet._getRshCommand(options, peer)
+                  cbackCommand = _ActionSet._getCbackCommand(options, peer)
+                  managedActions = _ActionSet._getManagedActions(options, peer)
+                  remotePeer = RemotePeer(peer.name, None, options.workingDir, remoteUser, None,
+                                          options.backupUser, rshCommand, cbackCommand)
+                  if managedActions is not None:
+                     for managedAction in managedActions:
+                        if managedAction in peerMap:
+                           if remotePeer not in peerMap[managedAction]:
+                              peerMap[managedAction].append(remotePeer)
+                        else:
+                           peerMap[managedAction] = [ remotePeer, ]
+      return peerMap
+   _buildPeerMap = staticmethod(_buildPeerMap)
+
    def _deriveHooks(action, preHookDict, postHookDict):
       """
       Derive pre- and post-action hooks, if any, associated with named action.
@@ -666,9 +823,62 @@ class _ActionSet(object):
 
       @raise Exception: If there is a problem executing the actions.
       """
+      logger.debug("Executing local actions.")
       for actionItem in self.actionSet:
          actionItem.executeAction(configPath, options, config)
 
+   def _getRemoteUser(options, remotePeer):
+      """
+      Gets the remote user associated with a remote peer.
+      Use peer's if possible, otherwise take from options section.
+      @param options: OptionsConfig object, as from config.options
+      @param remotePeer: Configuration-style remote peer object.
+      @return: Name of remote user associated with remote peer.
+      """
+      if remotePeer.remoteUser is None:
+         return options.backupUser
+      return remotePeer.remoteUser
+   _getRemoteUser = staticmethod(_getRemoteUser)
+
+   def _getRshCommand(options, remotePeer):
+      """
+      Gets the RSH command associated with a remote peer.
+      Use peer's if possible, otherwise take from options section.
+      @param options: OptionsConfig object, as from config.options
+      @param remotePeer: Configuration-style remote peer object.
+      @return: RSH command associated with remote peer.
+      """
+      if remotePeer.rshCommand is None:
+         return options.rshCommand
+      return remotePeer.rshCommand
+   _getRshCommand = staticmethod(_getRshCommand)
+
+   def _getCbackCommand(options, remotePeer):
+      """
+      Gets the cback command associated with a remote peer.
+      Use peer's if possible, otherwise take from options section.
+      @param options: OptionsConfig object, as from config.options
+      @param remotePeer: Configuration-style remote peer object.
+      @return: cback command associated with remote peer.
+      """
+      if remotePeer.cbackCommand is None:
+         return options.cbackCommand
+      return remotePeer.cbackCommand
+   _getCbackCommand = staticmethod(_getCbackCommand)
+
+   def _getManagedActions(options, remotePeer):
+      """
+      Gets the managed actions list associated with a remote peer.
+      Use peer's if possible, otherwise take from options section.
+      @param options: OptionsConfig object, as from config.options
+      @param remotePeer: Configuration-style remote peer object.
+      @return: Set of managed actions associated with remote peer.
+      """
+      if remotePeer.managedActions is None:
+         return options.managedActions
+      return remotePeer.managedActions
+   _getManagedActions = staticmethod(_getManagedActions)
+
 
 #######################################################################
 # Utility functions
@@ -689,29 +899,31 @@ def _usage(fd=sys.stderr):
    fd.write("\n")
    fd.write(" The following switches are accepted:\n")
    fd.write("\n")
-   fd.write("   -h, --help     Display this usage/help listing\n")
-   fd.write("   -V, --version  Display version information\n")
-   fd.write("   -b, --verbose  Print verbose output as well as logging to disk\n")
-   fd.write("   -q, --quiet    Run quietly (display no output to the screen)\n")
-   fd.write("   -c, --config   Path to config file (default: %s)\n" % DEFAULT_CONFIG)
-   fd.write("   -f, --full     Perform a full backup, regardless of configuration\n")
-   fd.write("   -l, --logfile  Path to logfile (default: %s)\n" % DEFAULT_LOGFILE)
-   fd.write("   -o, --owner    Logfile ownership, user:group (default: %s:%s)\n" % (DEFAULT_OWNERSHIP[0], DEFAULT_OWNERSHIP[1]))
-   fd.write("   -m, --mode     Octal logfile permissions mode (default: %o)\n" % DEFAULT_MODE)
-   fd.write("   -O, --output   Record some sub-command (i.e. cdrecord) output to the log\n")
-   fd.write("   -d, --debug    Write debugging information to the log (implies --output)\n")
-   fd.write("   -s, --stack    Dump a Python stack trace instead of swallowing exceptions\n")
+   fd.write("   -h, --help         Display this usage/help listing\n")
+   fd.write("   -V, --version      Display version information\n")
+   fd.write("   -b, --verbose      Print verbose output as well as logging to disk\n")
+   fd.write("   -q, --quiet        Run quietly (display no output to the screen)\n")
+   fd.write("   -c, --config       Path to config file (default: %s)\n" % DEFAULT_CONFIG)
+   fd.write("   -f, --full         Perform a full backup, regardless of configuration\n")
+   fd.write("   -M, --managed      Include managed clients when executing actions\n")
+   fd.write("   -N, --managed-only Include ONLY managed clients when executing actions\n")
+   fd.write("   -l, --logfile      Path to logfile (default: %s)\n" % DEFAULT_LOGFILE)
+   fd.write("   -o, --owner        Logfile ownership, user:group (default: %s:%s)\n" % (DEFAULT_OWNERSHIP[0], DEFAULT_OWNERSHIP[1]))
+   fd.write("   -m, --mode         Octal logfile permissions mode (default: %o)\n" % DEFAULT_MODE)
+   fd.write("   -O, --output       Record some sub-command (i.e. cdrecord) output to the log\n")
+   fd.write("   -d, --debug        Write debugging information to the log (implies --output)\n")
+   fd.write("   -s, --stack        Dump a Python stack trace instead of swallowing exceptions\n") # exactly 80 characters in width!
    fd.write("\n")
    fd.write(" The following actions may be specified:\n")
    fd.write("\n")
-   fd.write("   all            Take all normal actions (collect, stage, store, purge)\n")
-   fd.write("   collect        Take the collect action\n")
-   fd.write("   stage          Take the stage action\n")
-   fd.write("   store          Take the store action\n")
-   fd.write("   purge          Take the purge action\n")
-   fd.write("   rebuild        Rebuild \"this week's\" disc if possible\n")
-   fd.write("   validate       Validate configuration only\n")
-   fd.write("   initialize     Initialize media for use with Cedar Backup\n")
+   fd.write("   all                Take all normal actions (collect, stage, store, purge)\n")
+   fd.write("   collect            Take the collect action\n")
+   fd.write("   stage              Take the stage action\n")
+   fd.write("   store              Take the store action\n")
+   fd.write("   purge              Take the purge action\n")
+   fd.write("   rebuild            Rebuild \"this week's\" disc if possible\n")
+   fd.write("   validate           Validate configuration only\n")
+   fd.write("   initialize         Initialize media for use with Cedar Backup\n")
    fd.write("\n")
    fd.write(" You may also specify extended actions that have been defined in\n")
    fd.write(" configuration.\n")
@@ -1035,6 +1247,8 @@ class Options(object):
       self._quiet = False
       self._config = None
       self._full = False
+      self._managed = False
+      self._managedOnly = False
       self._logfile = None
       self._owner = None
       self._mode = None
@@ -1113,6 +1327,16 @@ class Options(object):
             return -1
          else:
             return 1
+      if self._managed != other._managed:
+         if self._managed < other._managed:
+            return -1
+         else:
+            return 1
+      if self._managedOnly != other._managedOnly:
+         if self._managedOnly < other._managedOnly:
+            return -1
+         else:
+            return 1
       if self._logfile != other._logfile:
          if self._logfile < other._logfile:
             return -1
@@ -1250,6 +1474,38 @@ class Options(object):
       """
       return self._full
 
+   def _setManaged(self, value):
+      """
+      Property target used to set the managed flag.
+      No validations, but we normalize the value to C{True} or C{False}.
+      """
+      if value:
+         self._managed = True
+      else:
+         self._managed = False
+
+   def _getManaged(self):
+      """
+      Property target used to get the managed flag.
+      """
+      return self._managed
+
+   def _setManagedOnly(self, value):
+      """
+      Property target used to set the managedOnly flag.
+      No validations, but we normalize the value to C{True} or C{False}.
+      """
+      if value:
+         self._managedOnly = True
+      else:
+         self._managedOnly = False
+
+   def _getManagedOnly(self):
+      """
+      Property target used to get the managedOnly flag.
+      """
+      return self._managedOnly
+
    def _setLogfile(self, value):
       """
       Property target used to set the logfile parameter.
@@ -1393,6 +1649,8 @@ class Options(object):
    quiet = property(_getQuiet, _setQuiet, None, "Command-line quiet (C{-q,--quiet}) flag.")
    config = property(_getConfig, _setConfig, None, "Command-line configuration file (C{-c,--config}) parameter.")
    full = property(_getFull, _setFull, None, "Command-line full-backup (C{-f,--full}) flag.")
+   managed = property(_getManaged, _setManaged, None, "Command-line managed (C{-M,--managed}) flag.")
+   managedOnly = property(_getManagedOnly, _setManagedOnly, None, "Command-line managed-only (C{-N,--managed-only}) flag.")
    logfile = property(_getLogfile, _setLogfile, None, "Command-line logfile (C{-l,--logfile}) parameter.")
    owner = property(_getOwner, _setOwner, None, "Command-line owner (C{-o,--owner}) parameter, as tuple C{(user,group)}.")
    mode = property(_getMode, _setMode, None, "Command-line mode (C{-m,--mode}) parameter.")
@@ -1423,6 +1681,8 @@ class Options(object):
       if not self.help and not self.version:
          if self.actions is None or len(self.actions) == 0:
             raise ValueError("At least one action must be specified.")
+      if self.managed and self.managedOnly:
+         raise ValueError("The --managed and --managed-only options may not be combined.")
 
    def buildArgumentList(self, validate=True):
       """
@@ -1467,6 +1727,10 @@ class Options(object):
          argumentList.append(self.config)
       if self.full:
          argumentList.append("--full")
+      if self.managed:
+         argumentList.append("--managed")
+      if self.managedOnly:
+         argumentList.append("--managed-only")
       if self.logfile is not None:
          argumentList.append("--logfile")
          argumentList.append(self.logfile)
@@ -1529,6 +1793,10 @@ class Options(object):
          argumentString += "--config \"%s\" " % self.config
       if self.full:
          argumentString += "--full "
+      if self.managed:
+         argumentString += "--managed "
+      if self.managedOnly:
+         argumentString += "--managed-only "
       if self.logfile is not None:
          argumentString += "--logfile \"%s\" " % self.logfile
       if self.owner is not None:
@@ -1585,6 +1853,10 @@ class Options(object):
          self.config = switches["--config"]
       if switches.has_key("-f") or switches.has_key("--full"):
          self.full = True
+      if switches.has_key("-M") or switches.has_key("--managed"):
+         self.managed = True
+      if switches.has_key("-N") or switches.has_key("--managed-only"):
+         self.managedOnly = True
       if switches.has_key("-l"):
          self.logfile = switches["-l"]
       if switches.has_key("--logfile"):
