@@ -11,7 +11,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: release.py 1217 2007-07-11 01:27:28Z pronovic $
+# Revision : $Id: release.py 826 2007-12-19 04:08:16Z pronovic $
 # Purpose  : Provides location to maintain release information.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -34,7 +34,7 @@ Provides location to maintain version information.
 AUTHOR      = "Kenneth J. Pronovici"
 EMAIL       = "pronovic@ieee.org"
 COPYRIGHT   = "2004-2007"
-VERSION     = "2.13.2"
-DATE        = "10 Jul 2007"
+VERSION     = "2.15.0"
+DATE        = "18 Dec 2007"
 URL         = "http://cedar-solutions.com/software/cedar-backup"
 
